"""
Created on 10 avril 2018

@author: Tonneau Stephane
"""
import pid_entry


class ProcessLog():
    """Class that allow the logging of the relevant process log information"""

    def __init__(self, pid, meta_information, logger):
        """
        Initialisator of the class

        :param pid: the standard information to log
        :param meta_information: meta information about the information to log
        :param logger: a list of Logging methods to use for logging
        :type entry: PidEntry
        :type meta_information: Metainformation
        :type logger: list<LoggingMethod>
        """
        self.pid_entry = pid
        self.meta_information = meta_information
        self.logging_methods = logger

    def __close_logging(self):
        """close the different logging systems

        :returns : the object itself
        :rtype: ProcessLog
        """
        for key in self.logging_methods:
            key.close()
        return self

    def __write_log(self):
        """write the logs in the different logging systems

        :returns : the object itself
        :rtype: ProcessLog
        """
        for key in self.logging_methods:
            key.process_log(self.pid_entry, self.meta_information)
        return self

    def add_file(self, path, context,
                 type_value=None, geometry=None,
                 checksum=None, size=None, product_id=None, ogc_urn=None):
        """add a file information to the log entry
        :param path: absolute path to the file or directory
        :param context: the context associated with the file
        :param type_value: the media type associated with the file
        :param geometry: the geometry in the GeoJSON format
        :param checksum: the checksum of the file in the format {[SHA|MD5|..]}<checksum value>
        :param size: the total size of the file or directory
        :param product_id: the product_id of the product to which the file or directory refers
        :param ogc_urn: the ogc urn reference of the product to which the file or directory refers
        :type path: string
        :type context: string
        :type type_value: string
        :type geometry: wkt geometry
        :type checksum: int
        :type size: string
        :type product_id: string
        :type ogc_urn: string

        :returns : the object itself
        :rtype: ProcessLog
        """
        file_info = pid_entry.FileInfo(path, context, type_value, geometry,
                                       checksum, size, product_id, ogc_urn)
        self.pid_entry.files.append(file_info)
        return self

    def proc_started(self):
        """write the logs in the different logging systems

        :returns : the object itself
        :rtype: ProcessLog
        """
        self.__write_log()
        return self

    def proc_stopped(self, exit_code, message=None):
        """update the end time and write the logs in the different logging systems

        :param exit_code: the exit code number of the process
        :param message: a string that gives more information on the end of the process
        :type exit_code: int
        :type message: string

        :returns : the object itself
        :rtype: ProcessLog
        """
        if message is not None:
            self.pid_entry.message = message
        if exit_code is not None:
            self.pid_entry.exit_code = str(exit_code)
        self.pid_entry.finalise()
        self.__write_log()
        self.__close_logging()
        return self
