"""
Created on 26 mars 2018

@author: Stephane Tonneau

"""

from __future__ import absolute_import
import os
import datetime
import pid_entry
import log_entry
import base_utils
from environment_factory import EnvironmentFactory

PID_CONFIG_PATH = "pidconfigpath"


def fix_for_test():
    """function to created the environment variables for testing
    """
    os.environ["pidclient.elasticsearch.index"] = "testprobav4"
    os.environ["pidclient.elasticsearch.type"] = "products4"
    os.environ[
        "pidclient.elasticsearch.hosts"] = "es1.vgt.vito.be,es2.vgt.vito.be,es3.vgt.vito.be"
    os.environ[
        "pidclient.kafka.brokers"] = "epod1.vgt.vito.be:6668,epod17.vgt.vito.be:6668"
    # os.environ["kafka.secure"] = "gssapi"   pid_test
    os.environ["pidclient.kafka.topic"] = "pid_test2_es"


class LoggingFactory():
    """Class that encapsulate logging methods"""

    def __init__(self, **logger_info):
        """Initialisator of the class

        :param logger_info: specific parameters of the logger class
        :type logger_info: a dictionary
        """
        self.logging_methods = []
        self.system_information = {}
        configuration = None
        config_path = None
        if PID_CONFIG_PATH in os.environ:
            config_path = os.environ[PID_CONFIG_PATH]
        if logger_info is not None:
            if "classes" in logger_info:
                self.logging_methods = logger_info["classes"]
            if PID_CONFIG_PATH in logger_info:
                config_path = logger_info[PID_CONFIG_PATH]
            if "sysinfo" in logger_info:
                self.system_information = logger_info["sysinfo"]
        if config_path is not None:
            configuration = base_utils.read_config(config_path)
        if not self.logging_methods:
            if "logger" in logger_info:
                logger_name = logger_info["logger"]
            else:
                logger_name = base_utils.get_config(
                    configuration, "main", "logger")
            if logger_name is None:
                logger_tab = ("console")
            else:
                logger_tab = logger_name.replace(" ","").split(",")
            parameters = {"configuration" : configuration}
            parameters.update(logger_info)
            if "file" in logger_tab:
                self.use_file(**parameters)
            if "elasticsearch" in logger_tab:
                self.use_elastic(**parameters)
            if "console" in logger_tab:
                self.use_console(**parameters)
            if "kafka" in logger_tab:
                self.use_kafka(**parameters)

    def use_file(self, **parameters):
        """declare the filesystem as logging system

        :param parameters: the general logging parameters
        :type parameters: a python dictionary

        :returns : the object itself
        :rtype: LoggingFactory
        """
        import pid_logging.file_log as file_log
        self.logging_methods.append(file_log.FileLogging(**parameters))
        return self

    def use_elastic(self, **parameters):
        """declare elastic search as logging system

        :param parameters: the general logging parameters
        :type parameters: a python dictionary

        :returns : the object itself
        :rtype: LoggingFactory
        """
        import pid_logging.elasticsearch_log as elasticsearch_log
        self.logging_methods.append(
            elasticsearch_log.ElasticsearchLogging(**parameters))
        return self

    def use_console(self, **parameters):
        """declare the console as logging system

        :param parameters: the general logging parameters
        :type parameters: a python dictionary

        :returns : the object itself
        :rtype: LoggingFactory
        """
        import pid_logging.console_log as console_log
        self.logging_methods.append(
            console_log.ConsoleLogging(**parameters))
        return self

    def use_kafka(self, **parameters):
        """declare the console as logging system

        :param parameters: the general logging parameters
        :type parameters: a python dictionary

        :returns : the object itself
        :rtype: LoggingFactory
        """
        import pid_logging.kafka_log as kafka_log
        self.logging_methods.append(kafka_log.KafkaLogging(**parameters))
        return self

    def get_logger(self, product_id, product_type, product_date):
        """return a new ProcessLog object

        :param product_id: the identifier of the new generated product
        :param product_type: the product type of the new generated product
        :param product_date: the creation date of the new generated product
        :type product_id: string
        :type product_type: string
        :type product_date: string

        :returns : the object itself
        :rtype: ProcessLog
        """
        entry = pid_entry.PidEntry(product_id, product_type, product_date)
        environment = EnvironmentFactory(**self.system_information).environment
        entry.configuration = environment.configuration
        entry.job_id = environment.job_id
        entry.job_type = environment.job_type
        entry.user = environment.user
        entry.host = environment.host
        entry.start_time = datetime.datetime.now()
        entry.environment = environment.environment_type
        meta_information = pid_entry.Metainformation()
        meta_information.uid = environment.get_uid()
        return log_entry.ProcessLog(entry, meta_information, self.logging_methods)
