"""
Created on 8 mars 2018

@author: Tonneau Stephane
"""
import datetime
from base_utils import JsonBased
from base_utils import GeoJSON

class Metainformation(JsonBased):
    """Class that encapsulate some metainformation of pid entries"""

    def __init__(self, uid=None):
        """
        Initialisator of the class

        :param uid: The unique ID of the object
        :type uid: string
        """
        self.uid = uid
        self.timestamp = None

class FileInfo(JsonBased):
    """Class that encapsulate a file information"""

    def __init__(self, path, context, type_value=None, geometry=None,
                 checksum=None, size=None, product_id=None, ogc_urn=None):
        """
        Initialisator of the class

        :param path: absolute path to the file or directory
        :param context: the context associated with the file
        :param type_value: the media type associated with the file
        :param geometry: the geometry in the GeoJSON format
        :param checksum: the checksum of the file in the format {[SHA|MD5|..]}<checksum value>
        :param size: the total size of the file or directory
        :param product_id: the product_id of the product to which the file or directory refers
        :param ogc_urn: the ogc urn reference of the product to which the file or directory refers
        :type path: string
        :type context: string
        :type type_value: string
        :type geometry: wkt geometry
        :type checksum: int
        :type size: string
        :type product_id: string
        :type ogc_urn: string
        """

        self.context = context
        self.type = type_value
        self.path = path
        if geometry is not None:
            self.geometry = GeoJSON(geometry)
        self.checksum = checksum
        self.size = size
        self.product_id = product_id
        self.ogc_urn = ogc_urn


class PidEntry(JsonBased):
    """Class that can be used to store the pertinent process log information"""

    def __init__(self, product_id, product_type, product_date, job_id=None,
                 proc_type=None, environment=None, start_time=None, user=None, host=None,
                 job_desc=None, job_type=None, parent_job_id=None, end_time=None,
                 exit_code=None, message=None):
        """
        Initialisator of the class

        :param product_id: id of the product that the process has manipulated
        :param product_type: Type of the product that the process has manipulated
        :param product_date: creation date of the product that the process has manipulated
        :param job_id: id of the process
        :param proc_type: type of job execute by the process
               Can be initialize, download, pretest, configure, process, posttest,
               quicklook, validate, archive, clean,...
        :param environment: environment on which the process was executed
        :param start_time: start time of the execution of the process
        :param user: user under which the process was executed
        :param host: hostname on which the process was executed"
        :param job_desc: Informal description of the process
        :param job_type: type of process executed by the process
               Can be a Yarn/Spark application ID (application_1234567),
               or a workflow job id (Airflow/Oozie/...)
        :param parent_job_id: id of the parent of the process
        :param end_time: end time of the execution of the process
        :param exit_code: exit code of the process
        :param message: message associated to the process
        :type product_id: string
        :type product_type: string
        :type product_date: date
        :type job_id: string
        :type proc_type: string
        :type environment: string
        :type start_time: date
        :type user: string
        :type host: string
        :type job_desc: string
        :type job_type: string
        :type parent_job_id: string
        :type end_time: date
        :type exit_code: int
        :type message: string
        """

        self.product_id = product_id
        self.product_type = product_type
        self.product_date = product_date
        self.job_desc = job_desc
        self.proc_type = proc_type
        self.parent_job_id = parent_job_id
        self.job_id = job_id
        self.job_type = job_type
        self.start_time = start_time
        self.end_time = end_time
        self.duration = None
        """duration of the process"""
        self.exit_code = exit_code
        self.message = message
        self.user = user
        self.host = host
        self.environment = environment
        self.files = []
        """file manipulated/created/... by the process"""
        self.configuration = None
        """Sparc configuration associated to the process"""
        self.id = None
        """Unique id of this entry (populated if required)"""

    def finalise(self):
        """
        update the current process log information by
        fixing the endtime and the duration.

        :returns : the object itself
        :rtype: PidEntry
        """
        self.end_time = datetime.datetime.now()
        self.duration = (
            self.end_time - self.start_time).total_seconds()
        return self
